/**************************************************************************************
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        uv_batcher.ms
Date:           06/24/11
Description:    This script applies uvw modifiers to multiple objects individually,
                as one, or same material objects as one.
***************************************************************************************/

try (destroyDialog uvw_batcher_rollout) catch()


struct UvwBatcher
(
    /**************************************************************************
    Returns a UVW modifier with properties 'user_settings'
    **************************************************************************/
    fn get_uvw_modifier user_settings =
    (
        local uvw_modifier = Uvwmap()
        
        uvw_modifier.maptype = user_settings[1] - 1
        uvw_modifier.cap = user_settings[2]
        uvw_modifier.length = user_settings[3]
        uvw_modifier.width = user_settings[4]
        uvw_modifier.height = user_settings[5]
        uvw_modifier.utile = user_settings[6]
        uvw_modifier.uflip = user_settings[7]
        uvw_modifier.vtile = user_settings[8]
        uvw_modifier.vflip = user_settings[9]
        uvw_modifier.wtile = user_settings[10]
        uvw_modifier.wflip = user_settings[11]
        uvw_modifier.channel = user_settings[12] - 1
        uvw_modifier.mapChannel = user_settings[13]
        uvw_modifier.axis = user_settings[14] - 1
        uvw_modifier.realWorldMapSize = user_settings[15]
        
        uvw_modifier --return value
    ), --end fn
    
    /**************************************************************************
    Returns the largest dimension from the bounding box of 'object_list'
    **************************************************************************/
    fn get_largest_dimension object_list =
    (
        local min_bounds = point3 0 0 0
        local max_bounds = point3 0 0 0
        local largest_value = 0.0
        
        if object_list.count > 0 do
        (
            --CRITICAL! Initialize starting bounds!
            object_bounds = nodeLocalBoundingBox object_list[1]
            min_bounds = object_bounds[1]
            max_bounds = object_bounds[2]
        
            for i = 2 to object_list.count do
            (
                object_bounds = nodeLocalBoundingBox object_list[i]
                if object_bounds[1].x < min_bounds.x do min_bounds.x = object_bounds[1].x
                if object_bounds[1].y < min_bounds.y do min_bounds.y = object_bounds[1].y
                if object_bounds[1].z < min_bounds.z do min_bounds.z = object_bounds[1].z
                if object_bounds[2].x > max_bounds.x do max_bounds.x = object_bounds[2].x
                if object_bounds[2].y > max_bounds.y do max_bounds.y = object_bounds[2].y
                if object_bounds[2].z > max_bounds.z do max_bounds.z = object_bounds[2].z
            ) --end for
            
            largest_value = amax #(abs(min_bounds.x - max_bounds.x), abs(min_bounds.y - max_bounds.y), abs(min_bounds.z - max_bounds.z))
        ) --end if
        
        largest_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns 'object_lst' separated into groups of objects with the same material
    **************************************************************************/
    fn get_objects_by_material object_list =
    (
        local object_materials = #()
        local object_groups = #()
        
        for current in object_list do
        (
            index = findItem object_materials current.material
            
            if index == 0 then
            (
                append object_materials current.material
                append object_groups #(current)
            ) --end if
            else
                append object_groups[index] current
        ) --end for
        
        object_groups --return value
    ), --end fn
    
    /**************************************************************************
    Applies a UVW modifier to 'object_list' based on 'uv_mode' 
    **************************************************************************/
    fn set_uvw_modifier object_list uv_mode user_settings fit_uv =
    (
        case uv_mode of
        (
            (1):    (
                        max create mode
                        
                        for current in object_list do
                        (
                            uvw_modifier = this.get_uvw_modifier user_settings
                            
                            if fit_uv do
                                uvw_modifier.length = uvw_modifier.width = uvw_modifier.height = this.get_largest_dimension #(current)
                            
                            addModifier current uvw_modifier
                        ) --end for
                    )--end case 1
            (2):    (
                        uvw_modifier = this.get_uvw_modifier user_settings
                            
                        if fit_uv do
                            uvw_modifier.length = uvw_modifier.width = uvw_modifier.height = this.get_largest_dimension object_list
                        
                        max modify mode
                        modPanel.addModToSelection uvw_modifier
                        max create mode
                    )--end case 2
            (3):    (
                        max create mode
                        
                        for current_group in (this.get_objects_by_material object_list) do
                        (
                            select current_group
                                
                            uvw_modifier = this.get_uvw_modifier user_settings
                            
                            if fit_uv do
                                uvw_modifier.length = uvw_modifier.width = uvw_modifier.height = this.get_largest_dimension current_group
                            
                            max modify mode
                            modPanel.addModToSelection uvw_modifier
                            max create mode
                        ) --end for
                    )--enc case 3
        )--end case
    ) --end fn
) --end struct


(--put script into local scope
    global uvw_batcher_rollout
    local uvw_batcher_ini_file = getDir #plugcfg + "\\UvwBatcher.ini"
    local uvw_batcher_rollout_pos
    
    /**********************************************************************
    UI creation
    **********************************************************************/
    rollout uvw_batcher_rollout "UV Batcher"
    (
        local batcher = UvwBatcher()
        
        /**********************************************************************
        Refreshes the enabled/disabled controls based on the user settings
        **********************************************************************/
        fn enable_controls =
        (
            local the_state = uvw_batcher_rollout.mapping_radio.state
            
            if the_state == 4 or the_state == 6 or the_state == 7 then
            (
                uvw_batcher_rollout.real_world_checkbox.checked = false
                uvw_batcher_rollout.real_world_checkbox.enabled = false
            ) --end if
            else
                uvw_batcher_rollout.real_world_checkbox.enabled = true
            
            the_state = uvw_batcher_rollout.real_world_checkbox.checked
            uvw_batcher_rollout.length_spinner.enabled = not the_state
            uvw_batcher_rollout.width_spinner.enabled = not the_state
            uvw_batcher_rollout.height_spinner.enabled = not the_state
            uvw_batcher_rollout.u_tile_spinner.enabled = not the_state
            uvw_batcher_rollout.v_tile_spinner.enabled = not the_state
            uvw_batcher_rollout.w_tile_spinner.enabled = not the_state
            uvw_batcher_rollout.square_fit_checkbox.enabled = not the_state
        )--end on
        
        groupBox mode_group "Add Modifier" pos:[5,5] width:200 height:65
            radioButtons mode_radio "" pos:[15,20] width:134 height:32 labels:#("To Each Object", "To Selection", "To Objects with the Same Material") columns:1
            
        groupBox mapping_group "Mapping" pos:[5,75] width:200 height:205
            radioButtons mapping_radio "" pos:[15,90] width:85 height:112 labels:#("Planar", "Cylindrical", "Spherical", "Shrink Wrap", "Box", "Face", "XYZ to UVW")
            checkbox cylinder_cap_checkbox "Cap" pos:[100,106] width:40 height:15
            
            label length_label "L:" pos:[15,200] width:15 height:15
            spinner length_spinner "" pos:[30,200] width:60 height:15 range:[0,999999,10]
            label width_label "W:" pos:[15,220] width:15 height:15
            spinner width_spinner "" pos:[30,220] width:60 height:15 range:[0,999999,10]
            label height_label "H:" pos:[15,240] width:15 height:15
            spinner height_spinner "" pos:[30,240] width:60 height:15 range:[0,999999,10]
            
            label u_tile_label "U:" pos:[100,200] width:15 height:15
            spinner u_tile_spinner "" pos:[115,200] width:40 height:15 range:[-999999,999999,1]
            checkbox u_tile_flip_checkbox "Flip" pos:[160,200] width:35 height:15
            label v_tile_label "V:" pos:[100,220] width:15 height:15
            spinner v_tile_spinner "" pos:[115,220] width:40 height:15 range:[-999999,999999,1]
            checkbox v_tile_flip_checkbox "Flip" pos:[160,220] width:35 height:15
            label w_tile_label "W:" pos:[100,240] width:15 height:15
            spinner w_tile_spinner "" pos:[115,240] width:40 height:15 range:[-999999,999999,1]
            checkbox w_tile_flip_checkbox "Flip" pos:[160,240] width:35 height:15
            
            checkbox real_world_checkbox "Real-World Map Size" pos:[15,260] width:130 height:15
            
        groupBox channel_group "Channel" pos:[5,285] width:200 height:50
            radioButtons channel_radio "" pos:[15,300] width:122 height:32 labels:#("Map Channel", "Vertex Color Channel")
            spinner channel_spinner "" pos:[105,300] width:38 height:16 range:[1,99,1] type:#integer
            
        groupBox align_group "Alignment" pos:[5,340] width:200 height:35
            radioButtons alignment_radio "" pos:[15,355] width:97 height:15 labels:#("X", "Y", "Z") columns:3 default:3
            checkbox square_fit_checkbox "Square Fit" pos:[120,355] width:75 height:15
            
        groupBox options_group "Options" pos:[5,380] width:200 height:55
            checkbox selection_only_checkbox "Selection Only" pos:[15,395] width:110 height:15 checked:true
            checkbox collapse_checkbox "Collapse Stack" pos:[15,415] width:110 height:15
            
        button apply_uvw_button "Add UVW Modifier" pos:[5,445] width:200 height:20
        label status_label "INFO: Standing by..." pos:[5,473] width:200 height:15
        
        /**********************************************************************
        UI open event handler. Prepares UI for first time open
        **********************************************************************/
        on uvw_batcher_rollout open do
        (
            local config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "Mode"
            if config_setting != "" do mapping_radio.state = config_setting as integer
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "Cap"
            if config_setting == "true" do cylinder_cap_checkbox.checked = true
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "Length"
            if config_setting != "" do length_spinner.value = config_setting as float
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "Width"
            if config_setting != "" do width_spinner.value = config_setting as float
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "Height"
            if config_setting != "" do height_spinner.value = config_setting as float
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "UTile"
            if config_setting != "" do u_tile_spinner.value = config_setting as float
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "UTileFlip"
            if config_setting == "true" do u_tile_flip_checkbox.checked = true
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "VTile"
            if config_setting != "" do v_tile_spinner.value = config_setting as float
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "VTileFlip"
            if config_setting == "true" do v_tile_flip_checkbox.checked = true
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "WTile"
            if config_setting != "" do w_tile_spinner.value = config_setting as float
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "WTileFlip"
            if config_setting == "true" do w_tile_flip_checkbox.checked = true
            
            config_setting = getIniSetting uvw_batcher_ini_file "Mapping" "RealWorld"
            if config_setting == "true" do real_world_checkbox.checked = true
            
            config_setting = getIniSetting uvw_batcher_ini_file "Channel" "Mode"
            if config_setting != "" do channel_radio.state = config_setting as integer
            
            config_setting = getIniSetting uvw_batcher_ini_file "Channel" "Channel"
            if config_setting != "" do channel_spinner.value = config_setting as integer
            
            config_setting = getIniSetting uvw_batcher_ini_file "Alignment" "Axis"
            if config_setting != "" do alignment_radio.state = config_setting as integer
            
            config_setting = getIniSetting uvw_batcher_ini_file "Alignment" "Fit"
            if config_setting == "true" do square_fit_checkbox.checked = true
            
            enable_controls()
        ) --end on
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on uvw_batcher_rollout moved pos do
            setIniSetting uvw_batcher_ini_file "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        UI close event handler. Stores settings for next launch
        **********************************************************************/
        on uvw_batcher_rollout close do
        (
            setIniSetting uvw_batcher_ini_file "Mapping" "Mode" (mapping_radio.state as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "Cap" (cylinder_cap_checkbox.checked as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "Length" (length_spinner.value as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "Width" (width_spinner.value as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "Height" (height_spinner.value as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "UTile" (u_tile_spinner.value as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "UTileFlip" (u_tile_flip_checkbox.checked as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "VTile" (v_tile_spinner.value as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "VTileFlip" (v_tile_flip_checkbox.checked as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "WTile" (w_tile_spinner.value as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "WTileFlip" (w_tile_flip_checkbox.checked as string)
            setIniSetting uvw_batcher_ini_file "Mapping" "RealWorld" (real_world_checkbox.checked as string)
            setIniSetting uvw_batcher_ini_file "Channel" "Mode" (channel_radio.state as string)
            setIniSetting uvw_batcher_ini_file "Channel" "Channel" (channel_spinner.value as string)
            setIniSetting uvw_batcher_ini_file "Alignment" "Axis" (alignment_radio.state as string)
            setIniSetting uvw_batcher_ini_file "Alignment" "Fit" (square_fit_checkbox.checked as string)
        ) --end on
            
        /**********************************************************************
        Mode changed event handler. Enables/Disables the selection only
        checkbox based on mode
        **********************************************************************/
        on mode_radio changed the_state do
        (
            if the_state == 2 then
                selection_only_checkbox.enabled = false
            else
                selection_only_checkbox.enabled = true
        )--end on
        
        /**********************************************************************
        Mapping changed event handler. Enables/Disables ui controls
        **********************************************************************/
        on mapping_radio changed the_state do
            enable_controls()
        
        /**********************************************************************
        Real world changed event handler. Enables/Disables ui controls
        **********************************************************************/
        on real_world_checkbox changed the_state do
            enable_controls()
        
        /**********************************************************************
        UV button event handler. Applies UVW modifiers to objects
        **********************************************************************/
        on apply_uvw_button pressed do
        (
            local uvw_objects = #()
            local uvw_settings = #()
            
            if mode_radio.state == 2 and selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                return false
            ) --end if
            
            if selection_only_checkbox.checked and selection.count == 0 do
            (
                status_label.text = "ERROR: No objects selected"
                return false
            ) --end if
            
            status_label.text = "INFO: UVW mapping objects. Please wait..."
            
            if selection_only_checkbox.checked then
                uvw_objects = selection as array
            else
                uvw_objects = geometry
            
            --initialize array size first
            uvw_settings[15] = real_world_checkbox.checked
            
            uvw_settings[1] = mapping_radio.state
            uvw_settings[2] = cylinder_cap_checkbox.checked
            uvw_settings[3] = length_spinner.value
            uvw_settings[4] = width_spinner.value
            uvw_settings[5] = height_spinner.value
            uvw_settings[6] = u_tile_spinner.value
            uvw_settings[7] = u_tile_flip_checkbox.checked
            uvw_settings[8] = v_tile_spinner.value
            uvw_settings[9] = v_tile_flip_checkbox.checked
            uvw_settings[10] = w_tile_spinner.value
            uvw_settings[11] = w_tile_flip_checkbox.checked
            uvw_settings[12] = channel_radio.state
            uvw_settings[13] = channel_spinner.value
            uvw_settings[14] = alignment_radio.state
            
            --if real world size then override settings
            if real_world_checkbox.checked then
            (
                uvw_settings[3] = 1.0
                uvw_settings[4] = 1.0
                uvw_settings[5] = 1.0
                uvw_settings[6] = 1.0
                uvw_settings[8] = 1.0
                uvw_settings[10] = 1.0
                batcher.set_uvw_modifier uvw_objects mode_radio.state uvw_settings false
            ) --end if
            else
                batcher.set_uvw_modifier uvw_objects mode_radio.state uvw_settings square_fit_checkbox.checked
            
            max create mode
            
            --restore user selection if needed
            if mode_radio.state == 3 and selection_only_checkbox.checked do
                select uvw_objects
            
            if collapse_checkbox.checked do collapseStack uvw_objects
            
            status_label.text = "INFO: UVW mapping complete"
        )--end on
    )--end rollout
    
    uvw_batcher_rollout_pos = execute (getIniSetting uvw_batcher_ini_file "Dialog" "Position")
    if uvw_batcher_rollout_pos == OK do uvw_batcher_rollout_pos = [100,100]
    if uvw_batcher_rollout_pos.x > sysinfo.desktopsize.x - 210 do
        uvw_batcher_rollout_pos.x = sysinfo.desktopsize.x - 215
    if uvw_batcher_rollout_pos.y > sysinfo.desktopsize.y - 550 do
        uvw_batcher_rollout_pos.y = sysinfo.desktopsize.y - 550
    if uvw_batcher_rollout_pos.x < 0 do uvw_batcher_rollout_pos.x = 0 
    if uvw_batcher_rollout_pos.y < 0 do uvw_batcher_rollout_pos.y = 0
    createDialog uvw_batcher_rollout 210 490 uvw_batcher_rollout_pos.x uvw_batcher_rollout_pos.y
)--end local script scope
